@@ -117,6 +117,8 @@ nma_menu_add_create_network_item (GtkWidget *menu, NMApplet *applet)
 {
 	GtkWidget *menu_item;
 	GtkWidget *label;
+	GError *error = NULL;
+	gboolean disabled;
 
 	menu_item = gtk_menu_item_new ();
 	label = gtk_label_new_with_mnemonic (_("Create _New Wireless Network..."));
@@ -125,6 +127,14 @@ nma_menu_add_create_network_item (GtkWidget *menu, NMApplet *applet)
 	gtk_widget_show_all (menu_item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 	g_signal_connect (menu_item, "activate", G_CALLBACK (new_network_activate_cb), applet);
+
+	/* FIXME: should really use PolicyKit and NM permissions here instead
+	 * using using GConf mandatory settings.  But this works for now.
+	 */
+	disabled = gconf_client_get_bool (applet->gconf_client, PREF_DISABLE_WIFI_CREATE, &error);
+	if (error || disabled)
+		gtk_widget_set_sensitive (GTK_WIDGET (menu_item), FALSE);
+	g_clear_error (&error);
 }
 
 
@@ -316,12 +326,13 @@ none:
 	return NULL;
 }
 
-static NMConnection *
+static gboolean
 wireless_new_auto_connection (NMDevice *device,
-                              NMApplet *applet,
-                              gpointer user_data)
+                              gpointer dclass_data,
+                              AppletNewAutoConnectionCallback callback,
+                              gpointer callback_data)
 {
-	WirelessMenuItemInfo *info = (WirelessMenuItemInfo *) user_data;
+	WirelessMenuItemInfo *info = (WirelessMenuItemInfo *) dclass_data;
 	NMConnection *connection = NULL;
 	NMSettingConnection *s_con = NULL;
 	NMSettingWireless *s_wireless = NULL;
@@ -337,7 +348,7 @@ wireless_new_auto_connection (NMDevice *device,
 
 	if (!info->ap) {
 		g_warning ("%s: AP not set", __func__);
-		return NULL;
+		return FALSE;
 	}
 
 	s_wireless = (NMSettingWireless *) nm_setting_wireless_new ();
@@ -356,8 +367,15 @@ wireless_new_auto_connection (NMDevice *device,
 	dev_caps = nm_device_wifi_get_capabilities (NM_DEVICE_WIFI (device));
 	s_wireless_sec = get_security_for_ap (info->ap, dev_caps, &supported, &s_8021x);
 	if (!supported) {
+		g_warning ("Unsupported AP configuration: dev_caps 0x%X, ap_flags 0x%X, "
+		           "wpa_flags 0x%X, rsn_flags 0x%x, mode %d",
+		           dev_caps,
+		           nm_access_point_get_flags (info->ap),
+		           nm_access_point_get_wpa_flags (info->ap),
+		           nm_access_point_get_rsn_flags (info->ap),
+		           nm_access_point_get_mode (info->ap));
 		g_object_unref (s_wireless);
-		goto out;
+		return FALSE;
 	} else if (s_wireless_sec)
 		g_object_set (s_wireless, NM_SETTING_WIRELESS_SEC, NM_SETTING_WIRELESS_SECURITY_SETTING_NAME, NULL);
 
@@ -387,8 +405,8 @@ wireless_new_auto_connection (NMDevice *device,
 
 	nm_connection_add_setting (connection, NM_SETTING (s_con));
 
-out:
-	return connection;
+	(*callback) (connection, TRUE, FALSE, callback_data);
+	return TRUE;
 }
 
 static void
@@ -998,7 +1016,7 @@ queue_avail_access_point_notification (NMDevice *device)
 	                           NULL))
 		return;
 
-	data->id = g_timeout_add (3000, idle_check_avail_access_point_notification, data);
+	data->id = g_timeout_add_seconds (3, idle_check_avail_access_point_notification, data);
 }
 
 static void
@@ -1306,6 +1324,7 @@ wireless_dialog_response_cb (GtkDialog *foo,
 	NMDevice *device = NULL;
 	NMAccessPoint *ap = NULL;
 	NMAGConfConnection *gconf_connection;
+	const char *service = NM_DBUS_SERVICE_USER_SETTINGS;
 
 	if (response != GTK_RESPONSE_OK)
 		goto done;
@@ -1330,6 +1349,12 @@ wireless_dialog_response_cb (GtkDialog *foo,
 	g_assert (connection);
 	g_assert (device);
 
+	/* If it's a system connection we just need to tell NM to activate it */
+	if (nm_connection_get_scope (connection) == NM_CONNECTION_SCOPE_SYSTEM) {
+		service = NM_DBUS_SERVICE_SYSTEM_SETTINGS;
+		goto activate;
+	}
+
 	gconf_connection = nma_gconf_settings_get_by_connection (applet->gconf_settings, connection);
 	if (gconf_connection) {
 		/* Not a new or system connection, save the updated settings to GConf */
@@ -1409,8 +1434,9 @@ wireless_dialog_response_cb (GtkDialog *foo,
 		}
 	}
 
+activate:
 	nm_client_activate_connection (applet->nm_client,
-	                               NM_DBUS_SERVICE_USER_SETTINGS,
+	                               service,
 	                               nm_connection_get_path (connection),
 	                               device,
 	                               ap ? nm_object_get_path (NM_OBJECT (ap)) : NULL,
