@@ -47,10 +47,16 @@ typedef struct {
 
 GType ce_page_wireless_get_type (void);
 
-CEPageWireless *ce_page_wireless_new (NMConnection *connection);
+CEPage *ce_page_wireless_new (NMConnection *connection, GtkWindow *parent, GError **error);
 
 /* Caller must free returned array */
 GByteArray *ce_page_wireless_get_ssid (CEPageWireless *self);
 
+
+void wifi_connection_new (GtkWindow *parent,
+                          PageNewConnectionResultFunc result_func,
+                          PageGetConnectionsFunc get_connections_func,
+                          gpointer user_data);
+
 #endif  /* __PAGE_WIRELESS_H__ */
 
