@@ -29,6 +29,7 @@
 #include <nm-gsm-device.h>
 #include <nm-cdma-device.h>
 #include <nm-access-point.h>
+#include <nm-settings.h>
 
 #include <nm-setting-connection.h>
 #include <nm-setting-wired.h>
@@ -43,44 +44,12 @@
 #include "utils.h"
 #include "gconf-helpers.h"
 
-/*
- * utils_bin2hexstr
- *
- * Convert a byte-array into a hexadecimal string.
- *
- * Code originally by Alex Larsson <alexl@redhat.com> and
- *  copyright Red Hat, Inc. under terms of the LGPL.
- *
- */
-char *
-utils_bin2hexstr (const char *bytes, int len, int final_len)
-{
-	static char	hex_digits[] = "0123456789abcdef";
-	char *		result;
-	int			i;
-
-	g_return_val_if_fail (bytes != NULL, NULL);
-	g_return_val_if_fail (len > 0, NULL);
-	g_return_val_if_fail (len < 256, NULL);	/* Arbitrary limit */
-
-	result = g_malloc0 (len * 2 + 1);
-	for (i = 0; i < len; i++)
-	{
-		result[2*i] = hex_digits[(bytes[i] >> 4) & 0xf];
-		result[2*i+1] = hex_digits[bytes[i] & 0xf];
-	}
-	/* Cut converted key off at the correct length for this cipher type */
-	if (final_len > -1)
-		result[final_len] = '\0';
-
-	return result;
-}
-
 static char *ignored_words[] = {
 	"Semiconductor",
 	"Components",
 	"Corporation",
 	"Communications",
+	"Company",
 	"Corp.",
 	"Corp",
 	"Co.",
@@ -114,7 +83,7 @@ static char *ignored_phrases[] = {
 	"Communication S.p.A.",
 	"Business Mobile Networks BV",
 	"Mobile Broadband Minicard Composite Device",
-	"Mobile AB",
+	"Mobile Communications AB",
 	NULL
 };
 
@@ -134,12 +103,11 @@ fixup_desc_string (const char *desc)
 
 	/* Attempt to shorten ID by ignoring certain phrases */
 	for (item = ignored_phrases; *item; item++) {
-		guint32 temp_len = strlen (temp);
 		guint32 ignored_len = strlen (*item);
 
 		p = strstr (temp, *item);
 		if (p)
-			memmove (p, p + ignored_len, temp_len - (p - temp));
+			memmove (p, p + ignored_len, strlen (p + ignored_len) + 1); /* +1 for the \0 */
 	}
 
 	/* Attmept to shorten ID by ignoring certain individual words */
@@ -859,3 +827,54 @@ utils_ether_ntop (const struct ether_addr *mac)
 	                        mac->ether_addr_octet[4], mac->ether_addr_octet[5]);
 }
 
+
+static void
+add_one_name (gpointer data, gpointer user_data)
+{
+	NMExportedConnection *exported = NM_EXPORTED_CONNECTION (data);
+	NMConnection *connection;
+	NMSettingConnection *s_con;
+	const char *id;
+	GSList **list = (GSList **) user_data;
+
+	connection = nm_exported_connection_get_connection (exported);
+	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+	id = nm_setting_connection_get_id (s_con);
+	g_assert (id);
+	*list = g_slist_append (*list, (gpointer) id);
+}
+
+char *
+utils_next_available_name (GSList *connections, const char *format)
+{
+	GSList *names = NULL, *iter;
+	char *cname = NULL;
+	int i = 0;
+
+	g_slist_foreach (connections, add_one_name, &names);
+
+	if (g_slist_length (names) == 0)
+		return g_strdup_printf (format, 1);
+
+	/* Find the next available unique connection name */
+	while (!cname && (i++ < 10000)) {
+		char *temp;
+		gboolean found = FALSE;
+
+		temp = g_strdup_printf (format, i);
+		for (iter = names; iter; iter = g_slist_next (iter)) {
+			if (!strcmp (iter->data, temp)) {
+				found = TRUE;
+				break;
+			}
+		}
+		if (!found)
+			cname = temp;
+		else
+			g_free (temp);
+	}
+
+	g_slist_free (names);
+	return cname;
+}
+
