@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* NetworkManager Wireless Applet -- Display wireless access points and allow user control
  *
  * Dan Williams <dcbw@redhat.com>
@@ -224,6 +225,14 @@ security_combo_changed_manually (GtkWidget *combo,
 	security_combo_changed (combo, user_data);
 }
 
+static gboolean
+is_system_connection (NMAWirelessDialog *self)
+{
+	NMAWirelessDialogPrivate *priv = NMA_WIRELESS_DIALOG_GET_PRIVATE (self);
+
+	return priv->connection && (nm_connection_get_scope (priv->connection) == NM_CONNECTION_SCOPE_SYSTEM);
+}
+
 static GByteArray *
 validate_dialog_ssid (NMAWirelessDialog *self)
 {
@@ -272,6 +281,10 @@ stuff_changed_cb (WirelessSecurity *sec, gpointer user_data)
 			g_byte_array_free (ssid, TRUE);
 	}
 
+	/* Assume system connections are valid so that we don't have to get secrets for them. */
+	if (is_system_connection (self))
+		valid = TRUE;
+
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (self), GTK_RESPONSE_OK, valid);
 }
 
@@ -307,6 +320,10 @@ ssid_entry_changed (GtkWidget *entry, gpointer user_data)
 	}
 
 out:
+	/* Assume system connections are valid so that we don't have to get secrets for them. */
+	if (is_system_connection (self))
+		valid = TRUE;
+
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (self), GTK_RESPONSE_OK, valid);
 }
 
@@ -996,7 +1013,11 @@ internal_init (NMAWirelessDialog *self,
 		priv->network_name_focus = TRUE;
 	}
 
-	gtk_dialog_set_response_sensitive (GTK_DIALOG (self), GTK_RESPONSE_OK, FALSE);
+	/* Assume system connections are valid so that we don't have to get secrets for them. */
+	if (is_system_connection (self))
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (self), GTK_RESPONSE_OK, TRUE);
+	else
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (self), GTK_RESPONSE_OK, FALSE);
 
 	if (!device_combo_init (self, specific_device)) {
 		g_warning ("No wireless devices available.");
@@ -1118,19 +1139,24 @@ nma_wireless_dialog_get_connection (NMAWirelessDialog *self,
 	} else
 		connection = g_object_ref (priv->connection);
 
-	/* Fill security */
-	model = gtk_combo_box_get_model (GTK_COMBO_BOX (priv->sec_combo));
-	gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->sec_combo), &iter);
-	gtk_tree_model_get (model, &iter, S_SEC_COLUMN, &sec, -1);
-	if (sec) {
-		wireless_security_fill_connection (sec, connection);
-		wireless_security_unref (sec);
-	} else {
-		/* Unencrypted */
-		s_wireless = NM_SETTING_WIRELESS (nm_connection_get_setting (connection, NM_TYPE_SETTING_WIRELESS));
-		g_assert (s_wireless);
+	/* Only update security information for user connections, as for system
+	 * connections the applet just needs to tell NM to activate it.
+	 */
+	if (!is_system_connection (self)) {
+		/* Fill security */
+		model = gtk_combo_box_get_model (GTK_COMBO_BOX (priv->sec_combo));
+		gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->sec_combo), &iter);
+		gtk_tree_model_get (model, &iter, S_SEC_COLUMN, &sec, -1);
+		if (sec) {
+			wireless_security_fill_connection (sec, connection);
+			wireless_security_unref (sec);
+		} else {
+			/* Unencrypted */
+			s_wireless = NM_SETTING_WIRELESS (nm_connection_get_setting (connection, NM_TYPE_SETTING_WIRELESS));
+			g_assert (s_wireless);
 
-		g_object_set (s_wireless, NM_SETTING_WIRELESS_SEC, NULL, NULL);
+			g_object_set (s_wireless, NM_SETTING_WIRELESS_SEC, NULL, NULL);
+		}
 	}
 
 	/* Fill device */
